@@ -1,10 +1,11 @@
-from zeno.api import distill, metric, model, ZenoOptions, ZenoParameters
+from zeno.api import distill, inference, metric, model, ZenoOptions, ZenoParameters
 from zeno.runner import zeno
 
 __all__ = [
     "model",
     "distill",
     "metric",
+    "inference",
     "ZenoOptions",
     "ZenoParameters",
     "zeno",
