@@ -7,10 +7,12 @@ import asyncio
 import os
 from typing import Dict, List, Union
 
+import gradio as gr  # type: ignore
 from fastapi import FastAPI, HTTPException, WebSocket
 from fastapi.routing import APIRoute
 from fastapi.staticfiles import StaticFiles
 
+from zeno.backend import ZenoBackend
 from zeno.classes.base import ZenoColumn
 from zeno.classes.classes import (
     ColorsProjectRequest,
@@ -26,7 +28,17 @@ from zeno.classes.metadata import HistogramBucket, HistogramRequest, StringFilte
 from zeno.classes.projection import Points2D, PointsColors
 from zeno.classes.report import Report
 from zeno.classes.slice import FilterPredicateGroup, Slice, SliceMetric
-from zeno.data_pipeline.zeno_backend import ZenoBackend
+from zeno.processing.histogram_processing import (
+    filter_by_string,
+    histogram_buckets,
+    histogram_counts,
+    histogram_metrics,
+)
+from zeno.processing.projection_processing import (
+    check_embed_exists,
+    project_into_2D,
+    projection_colors,
+)
 
 
 def custom_generate_unique_id(route: APIRoute):
@@ -59,6 +71,42 @@ def get_server(zeno: ZenoBackend):
         name="base",
     )
 
+    # If an inference function is provided, mount the gradio app.
+    if zeno.inference_function:
+        # The input_columns should map to the input_components.
+        input_components, output_components, input_columns = zeno.inference_function(
+            zeno.zeno_options
+        )
+        zeno.gradio_input_columns = input_columns
+
+        gradio_app = gr.Interface(
+            fn=zeno.single_inference,
+            inputs=[
+                gr.components.Dropdown(
+                    zeno.model_names, value=zeno.model_names[0], label="Model"
+                ),
+                *input_components,
+            ],
+            outputs=output_components,
+            css="""
+                    :root {
+                    --button-primary-background-base: #6a1b9a;
+                    --button-primary-background-hover: #d2bae9;
+                    --button-primary-text-color-base: white;
+                    --button-primary-text-color-hover: white;
+                    --button-primary-border-color-hover: #6a1b9a;
+                    --button-primary-border-color: #6a1b9a;
+                    }
+                """,
+            allow_flagging="never",
+            analytics_enabled=False,
+        )
+        api_app = gr.mount_gradio_app(
+            app=api_app,
+            blocks=gradio_app,
+            path="/gradio",
+        )
+
     @api_app.get("/settings", response_model=ZenoSettings, tags=["zeno"])
     def get_settings():
         return ZenoSettings(
@@ -68,6 +116,7 @@ def get_server(zeno: ZenoBackend):
             data_column=zeno.data_column,
             data_origin="/data/" if os.path.exists(zeno.data_path) else zeno.data_path,
             calculate_histogram_metrics=zeno.calculate_histogram_metrics,
+            inference_view=True if zeno.inference_function else False,
             samples=zeno.samples,
             totalSize=zeno.df.shape[0],
         )
@@ -108,11 +157,11 @@ def get_server(zeno: ZenoBackend):
         "/histograms", response_model=List[List[HistogramBucket]], tags=["zeno"]
     )
     def get_histogram_buckets(req: List[ZenoColumn]):
-        return zeno.get_histogram_buckets(req)
+        return histogram_buckets(zeno.df, req)
 
     @api_app.post("/histogram-counts", response_model=List[List[int]], tags=["zeno"])
     def calculate_histogram_counts(req: HistogramRequest):
-        return zeno.get_histogram_counts(req)
+        return histogram_counts(zeno.df, req)
 
     @api_app.post(
         "/histogram-metrics",
@@ -120,7 +169,7 @@ def get_server(zeno: ZenoBackend):
         tags=["zeno"],
     )
     def calculate_histogram_metrics(req: HistogramRequest):
-        return zeno.get_histogram_metrics(req)
+        return histogram_metrics(zeno.df, zeno.calculate_metric, req)
 
     @api_app.post("/slice", tags=["zeno"])
     def create_new_slice(req: Slice):
@@ -132,7 +181,7 @@ def get_server(zeno: ZenoBackend):
 
     @api_app.post("/string-filter", response_model=List[str], tags=["zeno"])
     def filter_string_metadata(req: StringFilterRequest):
-        filt_out = zeno.filter_string_metadata(req)
+        filt_out = filter_by_string(zeno.df, req)
         return filt_out
 
     @api_app.post("/slice-metrics", response_model=List[SliceMetric], tags=["zeno"])
@@ -144,15 +193,15 @@ def get_server(zeno: ZenoBackend):
         """Checks if embedding exists for a model.
         Returns the boolean True or False directly
         """
-        return zeno.embed_exists(model)
+        return check_embed_exists(zeno.df, model)
 
     @api_app.post("/embed-project", tags=["zeno"], response_model=Points2D)
     def project_embed_into_2D(req: EmbedProject2DRequest):
-        return zeno.project_embed_into_2D(req.model, req.column)
+        return project_into_2D(zeno.df, zeno.id_column, req.model, req.column)
 
     @api_app.post("/colors-project", tags=["zeno"], response_model=PointsColors)
     def get_projection_colors(req: ColorsProjectRequest):
-        return zeno.get_projection_colors(req.column)
+        return projection_colors(zeno.df, req.column)
 
     @api_app.post("/entry", tags=["zeno"], response_model=str)
     def get_df_row_entry(req: EntryRequest):
